@@ -20,7 +20,7 @@
 #
 #     https://www.nipreps.org/community/licensing/
 #
-"""Plotting tools shared across MRIQC and FMRIPREP."""
+"""Plotting tools shared across MRIQC and fMRIPrep."""
 
 import numpy as np
 import nibabel as nb
@@ -29,7 +29,7 @@ import pandas as pd
 import matplotlib.pyplot as plt
 from matplotlib import gridspec as mgs
 import matplotlib.cm as cm
-from matplotlib.colors import ListedColormap, Normalize
+from matplotlib.colors import Normalize
 from matplotlib.colorbar import ColorbarBase
 
 DINA4_LANDSCAPE = (11.69, 8.27)
@@ -38,52 +38,51 @@ DINA4_LANDSCAPE = (11.69, 8.27)
 class fMRIPlot:
     """Generates the fMRI Summary Plot."""
 
-    __slots__ = ("func_file", "mask_data", "tr", "seg_data", "confounds", "spikes")
+    __slots__ = (
+        "timeseries",
+        "segments",
+        "tr",
+        "confounds",
+        "spikes",
+        "nskip",
+        "sort_carpet",
+        "paired_carpet",
+    )
 
     def __init__(
         self,
-        func_file,
-        mask_file=None,
-        data=None,
+        timeseries,
+        segments,
+        confounds=None,
         conf_file=None,
-        seg_file=None,
         tr=None,
         usecols=None,
         units=None,
         vlines=None,
         spikes_files=None,
+        nskip=0,
+        sort_carpet=True,
+        paired_carpet=False,
     ):
-        func_img = nb.load(func_file)
-        self.func_file = func_file
-        self.tr = tr or _get_tr(func_img)
-        self.mask_data = None
-        self.seg_data = None
-
-        if not isinstance(func_img, nb.Cifti2Image):
-            self.mask_data = nb.fileslice.strided_scalar(
-                func_img.shape[:3], np.uint8(1)
-            )
-            if mask_file:
-                self.mask_data = np.asanyarray(nb.load(mask_file).dataobj).astype(
-                    "uint8"
-                )
-            if seg_file:
-                self.seg_data = np.asanyarray(nb.load(seg_file).dataobj)
+        self.timeseries = timeseries
+        self.segments = segments
+        self.tr = tr
+        self.nskip = nskip
+        self.sort_carpet = sort_carpet
+        self.paired_carpet = paired_carpet
 
         if units is None:
             units = {}
         if vlines is None:
             vlines = {}
         self.confounds = {}
-        if data is None and conf_file:
-            data = pd.read_csv(
-                conf_file, sep=r"[\t\s]+", usecols=usecols, index_col=False
-            )
+        if confounds is None and conf_file:
+            confounds = pd.read_csv(conf_file, sep=r"[\t\s]+", usecols=usecols, index_col=False)
 
-        if data is not None:
-            for name in data.columns.ravel():
+        if confounds is not None:
+            for name in confounds.columns:
                 self.confounds[name] = {
-                    "values": data[[name]].values.ravel().tolist(),
+                    "values": confounds[[name]].values.squeeze().tolist(),
                     "units": units.get(name),
                     "cutoff": vlines.get(name),
                 }
@@ -114,9 +113,7 @@ class fMRIPlot:
 
         grid_id = 0
         for tsz, name, iszs in self.spikes:
-            spikesplot(
-                tsz, title=name, outer_gs=grid[grid_id], tr=self.tr, zscored=iszs
-            )
+            spikesplot(tsz, title=name, outer_gs=grid[grid_id], tr=self.tr, zscored=iszs)
             grid_id += 1
 
         if self.confounds:
@@ -126,290 +123,233 @@ class fMRIPlot:
 
         for i, (name, kwargs) in enumerate(self.confounds.items()):
             tseries = kwargs.pop("values")
-            confoundplot(
-                tseries,
-                grid[grid_id],
-                tr=self.tr,
-                color=palette[i],
-                name=name,
-                **kwargs
-            )
+            confoundplot(tseries, grid[grid_id], tr=self.tr, color=palette[i], name=name, **kwargs)
             grid_id += 1
 
         plot_carpet(
-            self.func_file, atlaslabels=self.seg_data, subplot=grid[-1], tr=self.tr
+            self.timeseries,
+            segments=self.segments,
+            subplot=grid[-1],
+            tr=self.tr,
+            sort_rows=self.sort_carpet,
+            drop_trs=self.nskip,
+            cmap="paired" if self.paired_carpet else None,
         )
-        # spikesplot_cb([0.7, 0.78, 0.2, 0.008])
         return figure
 
 
 def plot_carpet(
-    func,
-    atlaslabels=None,
+    data,
+    segments=None,
+    cmap=None,
+    tr=None,
     detrend=True,
-    nskip=0,
-    size=(950, 800),
     subplot=None,
     title=None,
     output_file=None,
-    legend=False,
-    tr=None,
-    lut=None,
+    size=(900, 1200),
+    sort_rows="ward",
+    drop_trs=0,
+    legend=True,
 ):
     """
-    Plot an image representation of voxel intensities across time also know
-    as the "carpet plot" or "Power plot". See Jonathan Power Neuroimage
-    2017 Jul 1; 154:150-158.
+    Plot an image representation of voxel intensities across time.
+
+    This kind of plot is known as "carpet plot" or "Power plot".
+    See Jonathan Power Neuroimage 2017 Jul 1; 154:150-158.
 
     Parameters
     ----------
-
-        func : string or nibabel-image object
-            Path to NIfTI or CIFTI BOLD image, or a nibabel-image object
-        atlaslabels: ndarray, optional
-            A 3D array of integer labels from an atlas, resampled into ``img`` space.
-            Required if ``func`` is a NIfTI image.
-        detrend : boolean, optional
-            Detrend and standardize the data prior to plotting.
-        nskip : int, optional
-            Number of volumes at the beginning of the scan marked as nonsteady state.
-            Only used by volumetric NIfTI.
-        size : tuple, optional
-            Size of figure.
-        subplot : matplotlib Subplot, optional
-            Subplot to plot figure on.
-        title : string, optional
-            The title displayed on the figure.
-        output_file : string, or None, optional
-            The name of an image file to export the plot to. Valid extensions
-            are .png, .pdf, .svg. If output_file is not None, the plot
-            is saved to a file, and the display is closed.
-        legend : bool
-            Whether to render the average functional series with ``atlaslabels`` as
-            overlay.
-        tr : float , optional
-            Specify the TR, if specified it uses this value. If left as None,
-            # of frames is plotted instead of time.
-        lut : ndarray, optional
-            Look up table for segmentations
+    data : N x T :obj:`numpy.array`
+        The functional data to be plotted (*N* sampling locations by *T* timepoints).
+    segments: :obj:`dict`, optional
+        A mapping between segment labels (e.g., `"Left Cortex"`) and list of indexes
+        in the data array.
+    cmap : colormap
+        Overrides the generation of an automated colormap.
+    tr : float , optional
+        Specify the TR, if specified it uses this value. If left as None,
+        # of frames is plotted instead of time.
+    detrend : :obj:`bool`, optional
+        Detrend and standardize the data prior to plotting.
+    subplot : matplotlib subplot, optional
+        Subplot to plot figure on.
+    title : string, optional
+        The title displayed on the figure.
+    output_file : string, or None, optional
+        The name of an image file to export the plot to. Valid extensions
+        are .png, .pdf, .svg. If output_file is not None, the plot
+        is saved to a file, and the display is closed.
+    size : :obj:`tuple`
+        Maximum number of samples to plot (voxels, timepoints)
+    sort_rows : :obj:`str` or :obj:`False` or :obj:`None`
+        Apply a clustering algorithm to reorganize the rows of the carpet.
+        ``""``, ``False``, and ``None`` skip clustering sorting.
+        ``"linkage"`` uses linkage hierarchical clustering
+        :obj:`scipy.cluster.hierarchy.linkage`.
+        Any other value that Python evaluates to ``True`` will use the
+        default clustering, which is :obj:`sklearn.cluster.ward_tree`.
 
     """
-    epinii = None
-    segnii = None
-    nslices = None
-    img = nb.load(func) if isinstance(func, str) else func
-
-    if isinstance(img, nb.Cifti2Image):
-        assert (
-            img.nifti_header.get_intent()[0] == "ConnDenseSeries"
-        ), "Not a dense timeseries"
-
-        data = img.get_fdata().T
-        matrix = img.header.matrix
-        struct_map = {
-            "LEFT_CORTEX": 1,
-            "RIGHT_CORTEX": 2,
-            "SUBCORTICAL": 3,
-            "CEREBELLUM": 4,
+    if segments is None:
+        segments = {
+            "whole brain (voxels)": list(range(data.shape[0]))
         }
-        seg = np.zeros((data.shape[0],), dtype="uint32")
-        for bm in matrix.get_index_map(1).brain_models:
-            if "CORTEX" in bm.brain_structure:
-                lidx = (1, 2)["RIGHT" in bm.brain_structure]
-            elif "CEREBELLUM" in bm.brain_structure:
-                lidx = 4
-            else:
-                lidx = 3
-            index_final = bm.index_offset + bm.index_count
-            seg[bm.index_offset:index_final] = lidx
-        assert len(seg[seg < 1]) == 0, "Unassigned labels"
 
-        # Decimate data
-        data, seg = _decimate_data(data, seg, size)
-        # preserve as much continuity as possible
-        order = seg.argsort(kind="stable")
-
-        cmap = ListedColormap([cm.get_cmap("Paired").colors[i] for i in (1, 0, 7, 3)])
-        assert len(cmap.colors) == len(
-            struct_map
-        ), "Mismatch between expected # of structures and colors"
-
-        # ensure no legend for CIFTI
+    nsegments = len(segments)
+    if nsegments == 1:
         legend = False
 
-    else:  # Volumetric NIfTI
-        from nilearn._utils import check_niimg_4d
-        from nilearn._utils.niimg import _safe_get_data
+    if cmap is None:
+        colors = cm.get_cmap("tab10").colors
+    elif cmap == "paired":
+        colors = list(cm.get_cmap("Paired").colors)
+        colors[0], colors[1] = colors[1], colors[0]
+        colors[2], colors[7] = colors[7], colors[2]
 
-        img_nii = check_niimg_4d(img, dtype="auto",)
-        func_data = _safe_get_data(img_nii, ensure_finite=True)
-        func_data = func_data[..., nskip:]
-        ntsteps = func_data.shape[-1]
-        data = func_data[atlaslabels > 0].reshape(-1, ntsteps)
-        oseg = atlaslabels[atlaslabels > 0].reshape(-1)
-
-        # Map segmentation
-        if lut is None:
-            lut = np.zeros((256,), dtype="int")
-            lut[1:11] = 1
-            lut[255] = 2
-            lut[30:99] = 3
-            lut[100:201] = 4
-        # Apply lookup table
-        seg = lut[oseg.astype(int)]
-
-        # Decimate data
-        data, seg = _decimate_data(data, seg, size)
-        # Order following segmentation labels
-        order = np.argsort(seg)[::-1]
-        # Set colormap
-        cmap = ListedColormap(cm.get_cmap("tab10").colors[:4][::-1])
-
-        if legend:
-            epiavg = func_data.mean(3)
-            epinii = nb.Nifti1Image(epiavg, img_nii.affine, img_nii.header)
-            segnii = nb.Nifti1Image(
-                lut[atlaslabels.astype(int)], epinii.affine, epinii.header
-            )
-            segnii.set_data_dtype("uint8")
-            nslices = epiavg.shape[-1]
-
-    return _carpet(
-        data,
-        seg,
-        order,
-        cmap,
-        epinii=epinii,
-        segnii=segnii,
-        nslices=nslices,
-        tr=tr,
-        subplot=subplot,
-        legend=legend,
-        title=title,
-        output_file=output_file,
-    )
-
-
-def _carpet(
-    data,
-    seg,
-    order,
-    cmap,
-    tr=None,
-    detrend=True,
-    subplot=None,
-    legend=False,
-    title=None,
-    output_file=None,
-    epinii=None,
-    segnii=None,
-    nslices=None,
-):
-    """Common carpetplot building code for volumetric / CIFTI plots"""
-    from nilearn.plotting import plot_img
-    from nilearn.signal import clean
-
-    notr = False
-    if tr is None:
-        notr = True
-        tr = 1.0
-
-    # Detrend data
-    v = (None, None)
     if detrend:
-        data = clean(data.T, t_r=tr).T
-        v = (-2, 2)
+        from nilearn.signal import clean
+        data = clean(data.T, t_r=tr, filter=False).T
+
+    # We want all subplots to have the same dynamic range
+    vminmax = (np.percentile(data, 2), np.percentile(data, 98))
+
+    # Decimate number of time-series before clustering
+    n_dec = int((1.8 * data.shape[0]) // size[0])
+    if n_dec > 1:
+        segments = {
+            lab: idx[::n_dec] for lab, idx in segments.items()
+        }
+
+    # Cluster segments (if argument enabled)
+    if sort_rows:
+        from scipy.cluster.hierarchy import linkage, dendrogram
+        from sklearn.cluster import ward_tree
+
+        for seg_label, seg_idx in segments.items():
+            roi_data = data[seg_idx]
+            if isinstance(sort_rows, str) and sort_rows.lower() == "linkage":
+                linkage_matrix = linkage(
+                    roi_data, method="average", metric="euclidean", optimal_ordering=True
+                )
+            else:
+                children, _, n_leaves, _, distances = ward_tree(roi_data, return_distance=True)
+                linkage_matrix = _ward_to_linkage(children, n_leaves, distances)
+
+            dn = dendrogram(linkage_matrix, no_plot=True)
+            # Override the ordering of the indices in this segment
+            segments[seg_label] = np.array(seg_idx)[np.array(dn["leaves"])]
 
     # If subplot is not defined
     if subplot is None:
         subplot = mgs.GridSpec(1, 1)[0]
 
+    # Length before decimation
+    n_trs = data.shape[-1] - drop_trs
+
+    # Calculate time decimation factor
+    t_dec = max(int((1.8 * n_trs) // size[1]), 1)
+    data = data[:, drop_trs::t_dec]
+
     # Define nested GridSpec
-    wratios = [1, 100, 20]
     gs = mgs.GridSpecFromSubplotSpec(
+        nsegments,
         1,
-        2 + int(legend),
         subplot_spec=subplot,
-        width_ratios=wratios[: 2 + int(legend)],
-        wspace=0.0,
+        hspace=0.05,
+        height_ratios=[len(v) for v in segments.values()]
     )
 
-    # Segmentation colorbar
-    ax0 = plt.subplot(gs[0])
-    ax0.set_yticks([])
-    ax0.set_xticks([])
-    ax0.imshow(seg[order, np.newaxis], interpolation="none", aspect="auto", cmap=cmap)
+    for i, (label, indices) in enumerate(segments.items()):
+        # Carpet plot
+        ax = plt.subplot(gs[i])
 
-    ax0.grid(False)
-    ax0.spines["left"].set_visible(False)
-    ax0.spines["bottom"].set_color("none")
-    ax0.spines["bottom"].set_visible(False)
+        ax.imshow(
+            data[indices, :],
+            interpolation="nearest",
+            aspect="auto",
+            cmap="gray",
+            vmin=vminmax[0],
+            vmax=vminmax[1],
+        )
 
-    # Carpet plot
-    ax1 = plt.subplot(gs[1])
-    ax1.imshow(
-        data[order],
-        interpolation="nearest",
-        aspect="auto",
-        cmap="gray",
-        vmin=v[0],
-        vmax=v[1],
-    )
-
-    ax1.grid(False)
-    ax1.set_yticks([])
-    ax1.set_yticklabels([])
-
-    # Set 10 frame markers in X axis
-    interval = max((int(data.shape[-1] + 1) // 10, int(data.shape[-1] + 1) // 5, 1))
-    xticks = list(range(0, data.shape[-1])[::interval])
-    if notr:
-        xlabel = "time-points (index)"
-        xticklabels = [round(xtick) for xtick in xticks]
-    else:
-        xlabel = "time (s)"
-        xticklabels = ["%.02f" % (tr * xtick) for xtick in xticks]
-    ax1.set_xticks(xticks)
-    ax1.set_xlabel(xlabel)
-    ax1.set_xticklabels(xticklabels)
-
-    # Remove and redefine spines
-    for side in ["top", "right"]:
         # Toggle the spine objects
-        ax0.spines[side].set_color("none")
-        ax0.spines[side].set_visible(False)
-        ax1.spines[side].set_color("none")
-        ax1.spines[side].set_visible(False)
+        ax.spines["top"].set_color("none")
+        ax.spines["top"].set_visible(False)
+        ax.spines["right"].set_color("none")
+        ax.spines["right"].set_visible(False)
 
-    ax1.yaxis.set_ticks_position("left")
-    ax1.xaxis.set_ticks_position("bottom")
-    ax1.spines["bottom"].set_visible(False)
-    ax1.spines["left"].set_color("none")
-    ax1.spines["left"].set_visible(False)
-    if title:
-        ax1.set_title(title)
+        # Make colored left axis
+        ax.spines["left"].set_linewidth(3)
+        ax.spines["left"].set_color(colors[i])
+        ax.spines["left"].set_capstyle("butt")
+        ax.spines["left"].set_position(("outward", 2))
+
+        # Make all subplots have same xticks
+        xticks = np.linspace(0, data.shape[-1], endpoint=True, num=7)
+        ax.set_xticks(xticks)
+        ax.set_yticks([])
+        ax.grid(False)
+
+        if i == (nsegments - 1):
+            xlabel = "time-points (index)"
+            xticklabels = (xticks * n_trs / data.shape[-1]).astype("uint32") + drop_trs
+            if tr is not None:
+                xlabel = "time (mm:ss)"
+                xticklabels = [
+                    f"{int(t // 60):02d}:{(t % 60).round(0).astype(int):02d}"
+                    for t in (tr * xticklabels)
+                ]
+
+            ax.set_xlabel(xlabel)
+            ax.set_xticklabels(xticklabels)
+            ax.spines["bottom"].set_position(("outward", 5))
+            ax.spines["bottom"].set_color("k")
+            ax.spines["bottom"].set_linewidth(.8)
+        else:
+            ax.set_xticklabels([])
+            ax.set_xticks([])
+            ax.spines["bottom"].set_color("none")
+            ax.spines["bottom"].set_visible(False)
+
+        if title and i == 0:
+            ax.set_title(title)
+
+    if nsegments == 1:
+        ax.set_ylabel(label)
 
-    ax2 = None
     if legend:
-        gslegend = mgs.GridSpecFromSubplotSpec(
-            5, 1, subplot_spec=gs[2], wspace=0.0, hspace=0.0
+        from matplotlib.patches import Patch
+        from mpl_toolkits.axes_grid1.inset_locator import inset_axes
+
+        axlegend = inset_axes(
+            ax,
+            width="100%",
+            height=0.01,
+            loc='lower center',
+            borderpad=-4.1,
         )
-        coords = np.linspace(int(0.10 * nslices), int(0.95 * nslices), 5).astype(
-            np.uint8
+        axlegend.grid(False)
+        axlegend.set_xticks([])
+        axlegend.set_yticks([])
+        axlegend.patch.set_alpha(0.0)
+        for loc in ("top", "bottom", "left", "right"):
+            axlegend.spines[loc].set_color("none")
+            axlegend.spines[loc].set_visible(False)
+
+        axlegend.legend(
+            handles=[
+                Patch(color=colors[i], label=l)
+                for i, l in enumerate(segments.keys())
+            ],
+            loc="upper center",
+            bbox_to_anchor=(0.5, 0),
+            shadow=False,
+            fancybox=False,
+            ncol=min(len(segments.keys()), 5),
+            frameon=False,
+            prop={'size': 8}
         )
-        for i, c in enumerate(coords.tolist()):
-            ax2 = plt.subplot(gslegend[i])
-            plot_img(
-                segnii,
-                bg_img=epinii,
-                axes=ax2,
-                display_mode="z",
-                annotate=False,
-                cut_coords=[c],
-                threshold=0.1,
-                cmap=cmap,
-                interpolation="nearest",
-            )
 
     if output_file is not None:
         figure = plt.gcf()
@@ -418,7 +358,7 @@ def _carpet(
         figure = None
         return output_file
 
-    return (ax0, ax1, ax2), gs
+    return gs
 
 
 def spikesplot(
@@ -499,10 +439,7 @@ def spikesplot(
         )
 
         ytick_vals = np.arange(0.0, zs_max, float(np.floor(zs_max / 2.0)))
-        yticks = (
-            list(reversed((-1.0 * ytick_vals[ytick_vals > 0]).tolist()))
-            + ytick_vals.tolist()
-        )
+        yticks = list(reversed((-1.0 * ytick_vals[ytick_vals > 0]).tolist())) + ytick_vals.tolist()
 
         # TODO plot min/max or mark spikes
         # yticks.insert(0, ts_z.min())
@@ -521,9 +458,7 @@ def spikesplot(
             np.median(ts_z[:, nskip:]),
             ts_z[:, nskip:].max(),
         ]
-        ax.set_ylim(
-            0, max(yticks[-1] * 1.05, (yticks[-1] - yticks[0]) * 2.0 + yticks[-1])
-        )
+        ax.set_ylim(0, max(yticks[-1] * 1.05, (yticks[-1] - yticks[0]) * 2.0 + yticks[-1]))
         # ax.set_ylim(ts_z[:, nskip:].min() * 0.95,
         #             ts_z[:, nskip:].max() * 1.05)
 
@@ -624,9 +559,7 @@ def confoundplot(
     tseries = np.array(tseries)
 
     # Define nested GridSpec
-    gs = mgs.GridSpecFromSubplotSpec(
-        1, 2, subplot_spec=gs_ts, width_ratios=[1, 100], wspace=0.0
-    )
+    gs = mgs.GridSpecFromSubplotSpec(1, 2, subplot_spec=gs_ts, width_ratios=[1, 100], wspace=0.0)
 
     ax_ts = plt.subplot(gs[1])
     ax_ts.grid(False)
@@ -825,9 +758,7 @@ def compcor_variance_plot(
         if len(metadata_files) == 1:
             metadata_sources = ["CompCor"]
         else:
-            metadata_sources = [
-                "Decomposition {:d}".format(i) for i in range(len(metadata_files))
-            ]
+            metadata_sources = ["Decomposition {:d}".format(i) for i in range(len(metadata_files))]
     for file, source in zip(metadata_files, metadata_sources):
         metadata[source] = pd.read_csv(str(file), sep=r"\s+")
         metadata[source]["source"] = source
@@ -852,21 +783,14 @@ def compcor_variance_plot(
             decompositions.append((source, mask))
 
     if fig is not None:
-        ax = [
-            fig.add_subplot(1, len(decompositions), i + 1)
-            for i in range(len(decompositions))
-        ]
+        ax = [fig.add_subplot(1, len(decompositions), i + 1) for i in range(len(decompositions))]
     elif len(decompositions) > 1:
-        fig, ax = plt.subplots(
-            1, len(decompositions), figsize=(5 * len(decompositions), 5)
-        )
+        fig, ax = plt.subplots(1, len(decompositions), figsize=(5 * len(decompositions), 5))
     else:
         ax = [plt.axes()]
 
     for m, (source, mask) in enumerate(decompositions):
-        components = metadata[
-            (metadata["mask"] == mask) & (metadata["source"] == source)
-        ]
+        components = metadata[(metadata["mask"] == mask) & (metadata["source"] == source)]
         if len([m for s, m in decompositions if s == source]) > 1:
             title_mask = " ({} mask)".format(mask)
         else:
@@ -888,15 +812,11 @@ def compcor_variance_plot(
 
         for i, thr in enumerate(varexp_thresh):
             varexp[thr] = (
-                np.atleast_1d(
-                    np.searchsorted(components["cumulative_variance_explained"], thr)
-                )
+                np.atleast_1d(np.searchsorted(components["cumulative_variance_explained"], thr))
                 + 1
             )
             ax[m].axhline(y=100 * thr, color="lightgrey", linewidth=0.25)
-            ax[m].axvline(
-                x=varexp[thr], color="C{}".format(i), linewidth=2, linestyle=":"
-            )
+            ax[m].axvline(x=varexp[thr], color="C{}".format(i), linewidth=2, linestyle=":")
             ax[m].text(
                 0,
                 100 * thr,
@@ -907,9 +827,7 @@ def compcor_variance_plot(
             ax[m].text(
                 varexp[thr][0],
                 25,
-                "{} components explain\n{:.0f}% of variance".format(
-                    varexp[thr][0], 100 * thr
-                ),
+                "{} components explain\n{:.0f}% of variance".format(varexp[thr][0], 100 * thr),
                 rotation=90,
                 horizontalalignment="center",
                 fontsize="xx-small",
@@ -1057,7 +975,7 @@ def cifti_surfaces_plot(
     surface_type="inflated",
     clip_range=(0, None),
     output_file=None,
-    **splt_kwargs
+    **splt_kwargs,
 ):
     """
     Plots a CIFTI-2 dense timeseries onto left/right mesh surfaces.
@@ -1092,6 +1010,7 @@ def cifti_surfaces_plot(
 
     def get_surface_meshes(density, surface_type):
         import templateflow.api as tf
+
         lh, rh = tf.get("fsLR", density=density, suffix=surface_type, extension=[".surf.gii"])
         return str(lh), str(rh)
 
@@ -1130,10 +1049,7 @@ def cifti_surfaces_plot(
     # Build the figure
     lh_mesh, rh_mesh = get_surface_meshes(density, surface_type)
     p = splt.Plot(
-        surf_lh=lh_mesh,
-        surf_rh=rh_mesh,
-        layout=splt_kwargs.pop("layout", "row"),
-        **splt_kwargs
+        surf_lh=lh_mesh, surf_rh=rh_mesh, layout=splt_kwargs.pop("layout", "row"), **splt_kwargs
     )
     p.add_layer({'left': lh_data, 'right': rh_data}, cmap='YlOrRd_r')
     figure = p.build()  # figsize - leave default?
@@ -1147,50 +1063,6 @@ def cifti_surfaces_plot(
     return figure
 
 
-def _get_tr(img):
-    """
-    Attempt to extract repetition time from NIfTI/CIFTI header
-
-    Examples
-    --------
-    >>> _get_tr(nb.load(Path(test_data) /
-    ...    'sub-ds205s03_task-functionallocalizer_run-01_bold_volreg.nii.gz'))
-    2.2
-    >>> _get_tr(nb.load(Path(test_data) /
-    ...    'sub-01_task-mixedgamblestask_run-02_space-fsLR_den-91k_bold.dtseries.nii'))
-    2.0
-    """
-
-    try:
-        return img.header.matrix.get_index_map(0).series_step
-    except AttributeError:
-        return img.header.get_zooms()[-1]
-    raise RuntimeError("Could not extract TR - unknown data structure type")
-
-
-def _decimate_data(data, seg, size):
-    """Decimate timeseries data
-
-    Parameters
-    ----------
-    data : ndarray
-        2 element array of timepoints and samples
-    seg : ndarray
-        1 element array of samples
-    size : tuple
-        2 element for P/T decimation
-
-    """
-    p_dec = 1 + data.shape[0] // size[0]
-    if p_dec:
-        data = data[::p_dec, :]
-        seg = seg[::p_dec]
-    t_dec = 1 + data.shape[1] // size[1]
-    if t_dec:
-        data = data[:, ::t_dec]
-    return data, seg
-
-
 def _concat_brain_struct_data(structs, data):
     concat_data = np.array([], dtype=data.dtype)
     for struct in structs:
@@ -1198,3 +1070,17 @@ def _concat_brain_struct_data(structs, data):
         struct_data = data[struct.index_offset:struct_upper_bound]
         concat_data = np.concatenate((concat_data, struct_data))
     return concat_data
+
+
+def _ward_to_linkage(children, n_leaves, distances):
+    """Create linkage matrix from the output of Ward clustering."""
+    # create the counts of samples under each node
+    counts = np.zeros(children.shape[0])
+    n_samples = n_leaves
+    for i, merge in enumerate(children):
+        current_count = 0
+        for child_idx in merge:
+            current_count += 1 if child_idx < n_samples else counts[child_idx - n_samples]
+        counts[i] = current_count
+
+    return np.column_stack([children, distances, counts]).astype(float)
